import abstra_cli.apis.main as api_main
import abstra_cli.utils as utils


def list_workspace_vars():
    query = """
        query GetVars {
            environment_variables {
                name
                value
            }
        }
    """
    return api_main.hf_hasura_runner(query).get("environment_variables", [])


def add_workspace_vars(raw_vars):
    _, workspace_id, _ = api_main.get_auth_info()
    vars = [
        {"name": v["name"], "value": v["value"], "workspace_id": workspace_id}
        for v in raw_vars
    ]
    query = """
        mutation InsertVars($vars: [environment_variables_insert_input!]!) {
            insert_environment_variables(
                objects: $vars
                on_conflict: {
                    constraint: environment_variables_name_workspace_id_key
                    update_columns: [value, name]
                }
            ) {
                returning {
                    name
                    value
                }
            }
        }
    """
    return (
        api_main.hf_hasura_runner(query, {"vars": vars})
        .get("insert_environment_variables", {})
        .get("returning", [])
    )


def delete_workspace_vars(vars):
    query = """
        mutation DeleteVars($vars: [String!]) {
            delete_environment_variables(where: {name: {_in: $vars}}) {
                returning {
                    name
                    value
                }
            }
        }
    """
    return (
        api_main.hf_hasura_runner(query, {"vars": vars})
        .get("delete_environment_variables", {})
        .get("returning", [])
    )



import abstra_cli.apis.main as api_main
import abstra_cli.utils as utils


def list_workspace_dashes():
    query = """
        query GetDashes {
            dashes {
                path
                title
                script {
                    enabled
                }
            }
        }
    """
    return api_main.hf_hasura_runner(query).get("dashes", [])


def dash_upsert_data(deploy_data):
    return {
        "title": deploy_data["title"],
        "layout": deploy_data["layout"],
        "draft_layout": None,
        "theme": deploy_data.get("theme", None),
        "font_family": deploy_data.get("font_family", None),
        "main_color": deploy_data.get("main_color", None),
        "logo_url": deploy_data.get("logo_url", None),
        "brand_name": deploy_data.get("brand_name", None),
    }


def dash_script_upsert_data(deploy_data):
    return {
        "enabled": deploy_data.get("enabled", True),
        "file_path": deploy_data["code_file_path"],
        "code": "# using CODE_FILE_PATH",
        "draft": None,
        "name": deploy_data["title"],
    }


def add_workspace_dash(data):
    _, workspace_id, _ = api_main.get_auth_info()
    dash_data = {
        "path": data["path"],
        "workspace_id": workspace_id,
        **dash_upsert_data(data),
        "script": {
            "data": {
                "workspace_id": workspace_id,
                **dash_script_upsert_data(data),
            }
        },
    }

    query = """
        mutation InsertDash($dash_data: [dashes_insert_input!]!) {
            insert_dashes(
                objects: $dash_data
            ) {
                returning {
                    path
                    title
                }
            }
        }
    """

    return (
        api_main.hf_hasura_runner(query, {"dash_data": dash_data})
        .get("insert_dashes", {})
        .get("returning", {})[0]
    )


def update_workspace_dash(path, data):
    dash_data = dash_upsert_data(data)
    script_data = dash_script_upsert_data(data)

    request_data = {"path": path, "dash_data": dash_data, "script_data": script_data}
    update_query = """
        mutation UpdateDash($path: String!, $dash_data: dashes_set_input, $script_data: scripts_set_input = {}) {
            update_dashes(where: {path: {_eq: $path}}, _set: $dash_data) {
                returning {
                    id
                    path
                    title
                }
            }
            update_scripts(where: {dash: {path: {_eq: $path}}}, _set: $script_data) {
                returning {
                    name
                }
            }
        }
    """
    return (
        api_main.hf_hasura_runner(update_query, request_data)
        .get("update_dashes", {})
        .get("returning", {})[0]
    )


def upsert_workspace_dash(data):
    path = data["path"]

    query = """
        query FindDash($path: String!) {
            dashes(where: {path: {_eq: $path}}) {
                path
            }
        }
    """

    dashes = api_main.hf_hasura_runner(query, {"path": path}).get("dashes")
    if len(dashes):
        return update_workspace_dash(path, data)
    else:
        return add_workspace_dash(data)


def delete_workspace_dash(path):
    query = """
        mutation DeleteDash($path: String!) {
            delete_dashes(where: {path: {_eq: $path}}) {
                returning {
                    id
                    path
                    title
                }
            }
        }
    """

    return api_main.hf_hasura_runner(query, {"path": path})


def list_logs(limit, offset):
    query = """
        query GetDashLogs {
            dashes {
                id
                path
                logs {
                    id
                    created_at
                    execution_id
                    search_term
                    execution_type
                    messages
                }
            }
        }
    """
    dashes = api_main.hf_hasura_runner(query).get("dashes")
    logs = utils.flat_items_logs(dashes, path_or_id="path")

    return {"logs": utils.sampling(logs, limit, offset)}


def list_logs_by_path(path, limit, offset):
    query = """
        query GetDashLogs($limit: Int, $offset: Int, $path: String = "") {
            dashes(where: {path: {_eq: $path}}) {
                path
                logs(offset: $offset, limit: $limit, order_by: {created_at: desc}) {
                    id
                    created_at
                    dash_id
                    execution_id
                    search_term
                    execution_type
                    messages
                }
            }
        }
    """
    dashes = api_main.hf_hasura_runner(
        query, {"path": path, "limit": limit, "offset": offset}
    ).get("dashes")
    if dashes:
        return {"logs": utils.flatten_list([dash.get("logs") for dash in dashes])}
    return {"logs": []}



import abstra_cli.apis.main as api_main


def update_workspace_data(data: dict) -> dict:
    return {
        "sidebar": data["sidebar"],
    }


def get_workspace_id():
    query = """
        query GetWorkspaceId {
            workspaces {
                id
            }
        }
    """
    return (
        api_main.hf_hasura_runner(query, {}).get("workspaces", [{}])[0].get("id", None)
    )


def update_workspace(wid, data):
    workspace_data = update_workspace_data(data)

    request_data = {"wid": wid, "workspace_data": workspace_data}
    update_query = """
        mutation UpdateWorkspace($wid: uuid!, $workspace_data: workspaces_set_input) {
            update_workspaces(where: {id: {_eq: $wid}}, _set: $workspace_data) {
                returning {
                    id
                    sidebar
                }
            }
        }
    """
    return (
        api_main.hf_hasura_runner(update_query, request_data)
        .get("update_workspaces", {})
        .get("returning", {})[0]
    )



from . import parse_timestamp


def flat_dash_logs(dashes):
    """Ungroup dashes from the response of the list_logs query."""
    logs = []
    for dash in dashes:
        current_logs = dash.get("logs")
        for log in current_logs:
            log["path"] = dash.get("path")
            logs.append(log)

    logs.sort(key=lambda x: parse_timestamp(x["created_at"]), reverse=True)
    return logs



import requests, sys, json
from abstra_cli.resources.resources import Resource
import abstra_cli.messages as messages
import abstra_cli.utils as utils
import abstra_cli.apis as apis


NAME_PARAMETERS = ["name", "title"]
PATH_PARAMETERS = ["path"]
CODE_PARAMETERS = ["file", "f", "code", "c"]
NON_FLAG_PARAMETERS = NAME_PARAMETERS + PATH_PARAMETERS + CODE_PARAMETERS
FLAG_PARAMETERS = ["enabled"]
HOOK_PARAMETERS = FLAG_PARAMETERS + NON_FLAG_PARAMETERS


def check_valid_parameters(parameters: dict) -> None:
    for param in parameters.keys():
        if param not in HOOK_PARAMETERS:
            messages.invalid_parameter(param)
            sys.exit(1)
    for param, value in parameters.items():
        if param in NON_FLAG_PARAMETERS and value in [True, False]:
            messages.invalid_non_flag_parameter_value(param)
            sys.exit(1)


def evaluate_parameter_name(parameters: dict, use_default=True) -> dict:
    name = parameters.get("name") or parameters.get("n") or parameters.get("title")
    if not name and not use_default:
        return {}
    return {"name": name or "New Hook"}


def evaluate_parameter_path(parameters: dict) -> dict:
    path = parameters.get("path")
    if not path:
        return {}
    return {"path": path}


def evaluate_parameters_file_and_code(parameters: dict, use_default=True) -> dict:
    EMPTY_HOOk = "from abstra.hooks import get_request, send_json\nbody, query, headers = get_request()\nprint(body) # parsed json if content type is application/json, text otherwise\nprint(query, headers) # both dicts\nsend_json(data={'ok': True}, status_code=200, headers={'ABSTRA': 'CLOUD'})\n# send_response(body='plain text body')"
    file = parameters.get("file") or parameters.get("f")
    code = parameters.get("code") or parameters.get("c")

    if file and code:
        messages.code_and_file_not_allowed()
        sys.exit(1)

    if file:
        with open(file, "r") as f:
            return {"code": f.read()}

    if code:
        return {"code": code}

    return {"code": EMPTY_HOOk} if use_default else {}


def evaluate_flag_parameters(parameters: dict) -> dict:
    evaluated_params = {}
    for param, value in parameters.items():
        if param in FLAG_PARAMETERS:
            if value == "true" or value == True:
                evaluated_params[param] = True
                continue
            if value == "false" or value == False:
                evaluated_params[param] = False
                continue

            messages.invalid_flag_parameter_value(param)
            sys.exit(1)
    return evaluated_params


class Hooks(Resource):
    @staticmethod
    def list():
        hooks = apis.list_workspace_hooks()
        messages.print_hooks(hooks)

    @staticmethod
    def add(*args, **kwargs):
        upsert = kwargs.pop("upsert", False)
        path = kwargs.get("path")
        if upsert and not path:
            messages.upsert_without_identifier("path")
            sys.exit(1)

        check_valid_parameters(kwargs)

        hook_data = {
            **evaluate_parameter_name(kwargs),
            **evaluate_parameter_path(kwargs),
            **evaluate_parameters_file_and_code(kwargs),
            **evaluate_flag_parameters(kwargs),
        }

        if hook_data:
            try:
                if upsert:
                    apis.upsert_workspace_hook(hook_data)
                    messages.upserted_message("Hook", path)
                else:
                    path = apis.add_workspace_hook(hook_data)["path"]
                    messages.created_message("Hook", path)
            except Exception as e:
                print(e)
                messages.create_failed(
                    "Hook",
                )
                sys.exit(1)

    @staticmethod
    def update(*args, **kwargs):
        if not len(args):
            messages.required_argument("path")
            sys.exit(1)
        path = args[0]

        if not len(kwargs):
            messages.missing_parameters_to_update("hook", path)
            sys.exit(1)

        check_valid_parameters(kwargs)

        hook_data = {
            **evaluate_parameter_name(kwargs, use_default=False),
            **evaluate_parameter_path(kwargs),
            **evaluate_parameters_file_and_code(kwargs, use_default=False),
            **evaluate_flag_parameters(kwargs),
        }

        if hook_data:
            try:
                apis.update_workspace_hook(path, hook_data)
                messages.updated_message("Hook", path)
            except Exception as e:
                print(e)
                messages.update_failed("Hook", path)
                sys.exit(1)

    @staticmethod
    def remove(*args, **kwargs):
        if not len(args):
            messages.required_argument("path")
            sys.exit(1)

        path = args[0]
        apis.delete_workspace_hook(path)
        messages.deleted_message("Hook", path)

    @staticmethod
    def play(*args, **kwargs):
        # TODO: add body and headers
        if not len(args):
            messages.required_argument("path")
            sys.exit(1)

        method = kwargs.get("method") or kwargs.get("m") or "POST"
        path = args[0]

        subdomain_name = apis.get_subdomain()
        url = utils.get_prod_hook_url(subdomain_name, path)

        messages.hook_url(url, method)
        r = requests.request(method, url)
        print("Response:\n", r.text)

    @staticmethod
    def logs(*args, **kwargs):
        id = kwargs.get("id", None)
        limit = kwargs.get("limit", 20)
        offset = kwargs.get("offset", 0)

        if limit == 0:
            limit = None
        if id is None:
            logs = apis.hooks.list_logs(limit=limit, offset=offset)
        else:
            logs = apis.hooks.list_logs_by_id(id, limit=limit, offset=offset)

        serialized_logs = json.dumps(logs, default=str, indent=4)
        messages.print_logs(serialized_logs)



import sys
from abstra_cli.apis import get_subdomain, update_subdomain
from abstra_cli.resources.resources import Resource
import abstra_cli.messages as messages


class Subdomains(Resource):
    @staticmethod
    def list():
        subdomain = get_subdomain()
        messages.print_subdomains(subdomain)

    @staticmethod
    def update(*args, **kwargs):
        if not len(args):
            messages.missing_parameters_to_update("name", "subdomain")
            sys.exit(1)

        new_subdomain = args[0]
        old_subdomain = get_subdomain()
        try:
            updated_subdomain = update_subdomain(old_subdomain, new_subdomain)
            messages.print_subdomains(updated_subdomain[0]["name"])
        except Exception as error:
            if (
                len(error.args) > 0
                and len(error.args[0]) > 0
                and error.args[0][0]["extensions"]["code"] == "constraint-violation"
            ):
                messages.conflict_name("name", "subdomain")
            else:
                messages.update_failed("subdomain")
            sys.exit(1)



import abstra_cli.apis.main as api_main


def list_workspace_packages():
    query = """
        query GetPackages {
            packages {
                name
                version
            }
        }
    """
    return api_main.hf_hasura_runner(query).get("packages", [])


def add_workspace_packages(raw_packages):
    _, workspace_id, _ = api_main.get_auth_info()
    packages = [
        {"name": p["name"], "version": p["version"], "workspace_id": workspace_id}
        for p in raw_packages
    ]
    query = """
        mutation InsertPackages($packages: [packages_insert_input!]!) {
            insert_packages(
                objects: $packages
                on_conflict: {
                    constraint: packages_workspace_id_name_key  
                    update_columns: [version]
                }
            ) {
                returning {
                    name
                    version
                }
            }
        }
    """
    return (
        api_main.hf_hasura_runner(query, {"packages": packages})
        .get("insert_packages", {})
        .get("returning", [])
    )


def delete_workspace_packages(packages):
    query = """
        mutation DeletePackages($packages: [String!]) {
            delete_packages(where: {name: {_in: $packages}}) {
                returning {
                    name
                    version
                }
            }
        }
    """
    return (
        api_main.hf_hasura_runner(query, {"packages": packages})
        .get("delete_packages", {})
        .get("returning", [])
    )



import json, sys

from abstra_cli import messages
from abstra_cli.utils import parse_timestamp, sampling
from abstra_cli.resources.resources import Resource
from abstra_cli.apis import (
    jobs as jobs_api,
    hooks as hooks_api,
    forms as forms_api,
    dashes as dashes_api,
    workspaces as workspaces_api,
)

SIDEBAR_PARAMETERS = ["sidebar"]
NON_FLAG_PARAMETERS = SIDEBAR_PARAMETERS


def check_valid_parameters(parameters: dict) -> None:
    for param, value in parameters.items():
        if param in NON_FLAG_PARAMETERS and value in [True, False]:
            messages.invalid_non_flag_parameter_value(param)
            sys.exit(1)


def evaluate_optional_parameter(parameter_name: str, parameters: dict) -> dict:
    parameter_value = parameters.get(parameter_name)
    if not parameter_value:
        return {}
    return {parameter_name: parameter_value}


class Workspaces(Resource):
    @staticmethod
    def logs(*args, **kwargs):
        limit = kwargs.get("limit", None)
        offset = kwargs.get("offset", 0)

        job_logs = jobs_api.list_logs(limit, offset)["logs"]
        hook_logs = hooks_api.list_logs(limit, offset)["logs"]
        form_logs = forms_api.list_logs(limit, offset)["logs"]
        dash_logs = dashes_api.list_logs(limit, offset)["logs"]

        logs = job_logs + hook_logs + form_logs + dash_logs
        logs.sort(key=lambda x: parse_timestamp(x["created_at"]), reverse=True)
        logs = sampling(logs, limit, offset)
        serialized_logs = json.dumps(logs, default=str, indent=4)
        messages.print_logs(serialized_logs)

    @staticmethod
    def update(*args, **kwargs):
        wid = workspaces_api.get_workspace_id()
        if not wid:
            messages.error_getting_workspace_id()
            sys.exit(1)

        check_valid_parameters(kwargs)

        workspace_data = {
            **evaluate_optional_parameter("sidebar", kwargs),
        }

        if workspace_data:
            try:
                workspaces_api.update_workspace(wid, workspace_data)
                messages.updated_message("Workspace", wid)
            except Exception as e:
                print(e)
                messages.update_failed("Workspace", wid)
                sys.exit(1)

    @staticmethod
    def map_deploy_data(workspace_json_data: dict):
        return {
            "sidebar": workspace_json_data["workspace"].get("sidebar", None),
        }



import webbrowser, uuid, sys
import abstra_cli.messages as messages
import abstra_cli.credentials as credentials
import abstra_cli.apis.public as public_apis


def configure(api_token=None):
    api_token = api_token or messages.read_credentials()
    workspace_id, _ = public_apis.get_info_from_token(api_token)
    if not workspace_id:
        messages.invalid_credentials()
        sys.exit(1)

    credentials.save_credentials(api_token)
    print("Done!")


def login():
    cli_uuid = str(uuid.uuid4())
    url = f"https://forms.abstra.run/cli-login?uuid={cli_uuid}"

    opened = webbrowser.open(url)
    if not opened:
        messages.error_opening_browser(url)

    messages.waiting_for_api_token()
    api_token = public_apis.wait_for_api_token(cli_uuid)
    if not api_token:
        messages.failed_to_get_api_token()
        sys.exit(1)

    configure(api_token)
